/*************************************************************
 * i2cs_main.c
 *
 * I2C slav main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2010. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_I2CS
#include <port_api.h>
#include <mux_api.h>
#include <i2cs_api.h>

// I2C slave address
#define I2CS_MAIN_SLAVE_ADDRESS	0x01

// send data
#define I2CS_MAIN_SENDDATA		"FROM SLV"
#define I2CS_MAIN_RECVDATA_NUM	8

// I2C send data buffer state flag
#define	I2CS_MAIN_TXIDLE			0	// data transmit is not started
#define	I2CS_MAIN_TXSTART			1	// data transmit started
#define	I2CS_MAIN_TXSTOP			2	// data transmit stopped
static volatile unsigned char FlagTXStop;		// data transimit flag
static unsigned char *SendData;			// receive data
static unsigned char SendDataMax;		// receive data counter max
static volatile unsigned char SendDataCounter;	// receive data counter

// I2C receive data buffer state flag
#define	I2CS_MAIN_RXIDLE			0	// data receive is not started
#define	I2CS_MAIN_RXSTART			1	// data receive started
#define	I2CS_MAIN_RXSTOP			2	// data receive stopped
static volatile unsigned char FlagRXStop;		// data transmit flag
static unsigned char *RecvData;			// receive data
static unsigned char RecvDataMax;		// receive data counter max
static volatile unsigned char RecvDataCounter;	// receive data counter

// prototype
void i2cSlaveMain(void);
static void startI2Cslave(unsigned short address);
static void stopI2Cslave(void);
static unsigned char recvI2CslaveData(unsigned char *buf, unsigned char len);
static unsigned char sendI2CslaveData(unsigned char *buf, unsigned char len);
void i2cSlaveIntHandler(void)__attribute__((interrupt_handler));

/*
 * I2C slave sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 * 				2010/05/26	add pull-up setting
 */
void i2cSlaveMain(void)
{
	unsigned char	sendDataBuffer[] = I2CS_MAIN_SENDDATA;	// send data
	unsigned char	recvDataBuffer[I2CS_MAIN_RECVDATA_NUM];	// send data
	unsigned char*	buf;									// temporary pointer
	unsigned char	dataLength;								// actual data length
	MUX_USE_I2CSPORT use;									// set port

	printf("<<< I2C slave demonstration start >>>\n");

	// set slave address and start I2C slave
	startI2Cslave(I2CS_MAIN_SLAVE_ADDRESS);

	// P33,P34 pull up control disable
	PORT_controlPullup(PORT_NO_P3, PORT_PX0_ENABLE | PORT_PX1_ENABLE | PORT_PX2_ENABLE | PORT_PX3_DISABLE |
									PORT_PX4_DISABLE | PORT_PX5_ENABLE | PORT_PX6_ENABLE | PORT_PX7_ENABLE);

	printf("Received data : ");

	// receive data from I2C master
	dataLength = recvI2CslaveData(&recvDataBuffer[0], I2CS_MAIN_RECVDATA_NUM);

	buf = &recvDataBuffer[0];
	while(dataLength > 0) {
		printf("%c", *buf);
		buf++;
		dataLength--;
	}
	printf("\n");

	printf("Transmitted data : ");

	// send data to I2C master
	dataLength = sendI2CslaveData(&sendDataBuffer[0], (sizeof(sendDataBuffer) - 1 ));

	buf = &sendDataBuffer[0];
	while(dataLength > 0) {
		printf("%c", *buf);
		buf++;
		dataLength--;
	}
	printf("\n");

	// stop I2C slave
	stopI2Cslave();

	// clear port
#if	defined(S1C17624) || defined(S1C17622)
	use.selectNo = MUX_I2CS_0;
#endif // end of S1C17624/S1C17622
	use.sw = MUX_OFF;
	MUX_setI2CSport(&use);

	// P33,P34 pull up control enable
	PORT_controlPullup(PORT_NO_P3, PORT_PX0_ENABLE | PORT_PX1_ENABLE | PORT_PX2_ENABLE | PORT_PX3_ENABLE |
									PORT_PX4_ENABLE | PORT_PX5_ENABLE | PORT_PX6_ENABLE | PORT_PX7_ENABLE);

	printf("<<< I2C slave demonstration finish >>>\n");
}

/*
 * start I2C slave.
 * param	unsigned short address					slave address
 * return	void
 * history	:	2009/12/25
 */
static void startI2Cslave(unsigned short address)
{
	FlagTXStop = I2CS_MAIN_TXIDLE;
	FlagRXStop = I2CS_MAIN_RXIDLE;
	RecvData = 0;
	RecvDataCounter = 0;
	I2CS_setBusFreeReq(I2CS_BUSFREE_DISABLE);
	I2CS_init((unsigned char)address, I2CS_NOISEREMOVE_ON, I2CS_CLKSTRETCH_ON);

	I2CS_initInt(I2CS_INT_LEVEL3);
	I2CS_controlInt(I2CS_RXINT_ENABLE | I2CS_TXINT_ENABLE | I2CS_BUSINT_ENABLE);
	I2CS_setEnable(I2CS_CTL_ENABLE);
}

/*
 * stop I2C slave.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopI2Cslave(void)
{
	I2CS_setComEnable(I2CS_COM_DISABLE);

	I2CS_setEnable(I2CS_CTL_DISABLE);
	I2CS_controlInt(I2CS_RXINT_DISABLE | I2CS_TXINT_DISABLE | I2CS_BUSINT_DISABLE);
	I2CS_initInt(I2CS_INT_LEVEL0);
}

/*
 * start I2C slave data receive.
 * param	unsigned char  *buf      receive data buffler
 * param	unsigned char  len       receive data buffer length
 * return	unsigned char            actual received data length
 * history	:	2009/12/25
 */
static unsigned char recvI2CslaveData(unsigned char *buf, unsigned char len)
{
	unsigned char	valRet;			// actual receive data length

	RecvData = buf;
	RecvDataMax = len;
	RecvDataCounter = 0;

	I2CS_setComEnable(I2CS_COM_ENABLE);

	while(FlagRXStop != I2CS_MAIN_RXSTOP) {
		;				// wait stop condition
	}

	I2CS_setComEnable(I2CS_COM_DISABLE);

	valRet = RecvDataCounter;

	FlagRXStop = I2CS_MAIN_RXIDLE;
	RecvData = 0;
	RecvDataMax = 0;
	RecvDataCounter = 0;

	return valRet;
}

/*
 * start I2C slave data send.
 * param	unsigned char  *buf      send data buffler
 * param	unsigned char  len       send data length
 * return	unsigned char            actual sent data length
 * history	:	2009/12/25
 */
static unsigned char sendI2CslaveData(unsigned char *buf, unsigned char len)
{
	unsigned char	valRet;			// actual receive data length

	SendData = buf;
	SendDataMax = len;
	SendDataCounter = 0;

	I2CS_setComEnable(I2CS_COM_ENABLE);

	FlagTXStop = I2CS_MAIN_TXSTART;

	while(FlagTXStop != I2CS_MAIN_TXSTOP) {
		;				// wait stop condition
	}

	I2CS_setComEnable(I2CS_COM_DISABLE);

	valRet = SendDataCounter;

	FlagTXStop = I2CS_MAIN_TXIDLE;
	SendData = 0;
	SendDataMax = 0;
	SendDataCounter = 0;

	return valRet;
}

/*
 * I2C slave interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void i2cSlaveIntHandler(void)
{
	unsigned short	flgRxTxIntr;							// RX/TX interrupt flag
	unsigned short	flgBusIntr;								// bus status interrupt flag
	unsigned char	flgClearBusIntr;						// reset interrupt flag

	// get interrupt flag
	I2CS_checkIntFlag(&flgRxTxIntr);
	I2CS_checkBusStatusIntFlag(&flgBusIntr);

	// data receive
	if((flgRxTxIntr & I2CS_RXDATA_READY) == I2CS_RXDATA_READY) {
		if(FlagRXStop == I2CS_MAIN_RXIDLE) {
			FlagRXStop = I2CS_MAIN_RXSTART;
		}
		I2CS_getReceiveData(RecvData);
		if(RecvDataCounter < RecvDataMax) {
			RecvData++;
			RecvDataCounter++;
		}
	}

	// data send
	if((flgRxTxIntr & I2CS_TXDATA_EMPTY) == I2CS_TXDATA_EMPTY) {
		if(FlagTXStop == I2CS_MAIN_TXIDLE) {
			FlagTXStop = I2CS_MAIN_TXSTART;
		}
		if(SendDataCounter < SendDataMax) {
			I2CS_setTransmitData(*SendData);
			SendData++;
			SendDataCounter++;
		} else {
			I2CS_setClockStretch(I2CS_CLKSTRETCH_OFF);
		}
	}

	// bus status changed
	if((flgBusIntr & I2CS_BUSSTS_CHANGED) == I2CS_BUSSTS_CHANGED) {
		flgClearBusIntr = 0;

		if((flgBusIntr & I2CS_UDFOVF_OCCURRED) == I2CS_UDFOVF_OCCURRED) {
			flgClearBusIntr |= I2CS_UDFOVF_RESET;
		}

		if((flgBusIntr & I2CS_BUSFREE_OCCURRED) == I2CS_BUSFREE_OCCURRED) {
			flgClearBusIntr |= I2CS_BUSFREE_RESET;
		}

		if((flgBusIntr & I2CS_MISMATCH_ERROR) == I2CS_MISMATCH_ERROR) {
			flgClearBusIntr |= I2CS_MISMATCH_RESET;
		}

		if((flgBusIntr & I2CS_ASYNC_DETECTED) == I2CS_ASYNC_DETECTED) {
			flgClearBusIntr |= I2CS_ASYNC_RESET;
		}

		if((flgBusIntr & I2CS_RESPONSE_NAK) == I2CS_RESPONSE_NAK) {
			flgClearBusIntr |= I2CS_RESPONSE_RESET;
		}

		// stop condtion
		if((flgBusIntr & I2CS_STOP_DETECTED) == I2CS_STOP_DETECTED) {
			flgClearBusIntr |= I2CS_STOP_RESET;
			FlagTXStop = I2CS_MAIN_TXSTOP;
			FlagRXStop = I2CS_MAIN_RXSTOP;
		}
		I2CS_resetIntFlag(flgClearBusIntr);
	}
}
#endif	// end of PE_I2CS
