/*************************************************************
 * rtc_main.c
 *
 * real time clock main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2010. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_RTC
#include <rtc_api.h>
#include <osc_api.h>
#include "main.h"

#define	STR_BUFF_SIZE	50
#define	RTC_MENU_MAX	80
#define	STR_CHANGE_OFFSET	0x30
#define	PREFIX_LENGTH	2

static volatile unsigned char rtcIntCt;		// interrupt counter

/* prototype */
void rtcMain(void);
static void startRTC(void);
static void stopRTC(void);
static void getRTC(void);
static void setRTC(void);
static void countRTCInt(void);
static unsigned char rtcMenuDisp(void);
static unsigned char rtcMenuInput(unsigned char menuMax);
static void rtcDummy(void);
void rtcIntHandler(void)__attribute__ ((interrupt_handler));
void rtcInt(void);


// sub menu table
static MENU_TBL rtcMainMenu[] = {
	{ "get RTC                          "	, getRTC					},
	{ "set RTC                          "	, setRTC					},
	{ "indicate the count of interrupt  "	, countRTCInt				},
	{ "exit                             "	, rtcDummy					},
	{ NULL									, NULL						}
};

/*
 * RTC sample program
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void rtcMain(void)
{

	unsigned char	menuMax = 0xff;							// MAX menu number
	unsigned char	inputNo = 0x00;								// key input number
	// initialize interrupt count value
	rtcIntCt = 0;

	printf("<<< Real Time Clock demonstration start >>>\n");

	// start RTC
	startRTC();

	while(inputNo != menuMax) {
		// menu display
		menuMax = rtcMenuDisp();

		 // wait input menu number
		inputNo = rtcMenuInput(menuMax);

		 // select peripheral test
		if(inputNo != MENU_ERROR) {
			rtcMainMenu[inputNo-1U].function();
		}
	}

	// stop RTC
	stopRTC();

	printf("<<< Real Time Clock demonstration finish >>>\n");
}


/*
 * start RTC for 24H mode.
 * param	void
 * return	void
 * history	:	2010/04/05
 */
static void startRTC(void)
{
	RTC_DATE_INFO rtcDate;						// RTC count data

	rtcDate.year		= 0x00;
	rtcDate.month		= 0x01;
	rtcDate.day			= 0x01;
	rtcDate.hour 		= 0x00;
	rtcDate.minute 		= 0x00;
	rtcDate.second 		= 0x00;
	rtcDate.week 		= 0x00;

	// set RTC clock enable
	CLG_OSC_controlRTC(CLG_OSC_RTCCLK_ENABLE);

	// disable RTC interrupt
	RTC_controlInt(RTC_INT_DISABLE);

	// initialize RTC setting
	RTC_init(RTC_MODE_24H);

	// RTC time set
	RTC_setTime(&rtcDate);

	// initialize RTC interrupt setting
	RTC_initInt(RTC_INT_LEVEL3, RTC_INT_CYCLE_MINUTE, RTC_INT_MODE_LEVEL);

	// enable RTC interrupt
	RTC_controlInt(RTC_INT_ENABLE);

	// start RTC count
	RTC_setTimerRun(RTC_TIMER_RUN);
}


/*
 * stop RTC.
 * param	void
 * return	void
 * history	:	2010/04/05
 */
static void stopRTC(void)
{
	// stop RTC count
	RTC_setTimerRun(RTC_TIMER_STOP);

	// disable RTC interrupt
	RTC_controlInt(RTC_INT_DISABLE);

	// clear RTC interrupt level
	RTC_initInt(RTC_INT_LEVEL0, RTC_INT_CYCLE_SECOND, RTC_INT_MODE_LEVEL);

	// set RTC clock disable
	CLG_OSC_controlRTC(CLG_OSC_RTCCLK_DISABLE);
}


/*
 * get RTC time.
 * param	void
 * return	void
 * history	:	2010/04/05
 */
static void getRTC(void)
{
	RTC_DATE_INFO rtcDate;						// RTC count data
	unsigned char *pStr;
	unsigned short i;
	unsigned char	strBuff[STR_BUFF_SIZE];

	// get RTC count data
	RTC_getTime(&rtcDate);

	pStr = strBuff;
	for(i = 0; i < STR_BUFF_SIZE; i++) {
		strBuff[i] = '\0';
	}

	// get year data
	*pStr = STR_CHANGE_OFFSET + ((rtcDate.year >> 4) & 0x0f);
	pStr++;
	*pStr = STR_CHANGE_OFFSET + (rtcDate.year & 0x0f);
	pStr++;
	*pStr = '/';
	pStr++;

	// get month data
	*pStr = STR_CHANGE_OFFSET + ((rtcDate.month >> 4) & 0x01);
	pStr++;
	*pStr = STR_CHANGE_OFFSET + (rtcDate.month & 0x0f);
	pStr++;
	*pStr = '/';
	pStr++;

	// get day data
	*pStr = STR_CHANGE_OFFSET + ((rtcDate.day >> 4) & 0x03);
	pStr++;
	*pStr = STR_CHANGE_OFFSET + (rtcDate.day & 0x0f);
	pStr++;

	// get week data
	switch(rtcDate.week) {
		case RTC_WEEK_SUNDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'S';	pStr++;
			*pStr	= 'u';	pStr++;
			*pStr	= 'n';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		case RTC_WEEK_MONDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'M';	pStr++;
			*pStr	= 'o';	pStr++;
			*pStr	= 'n';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		case RTC_WEEK_TUESDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'T';	pStr++;
			*pStr	= 'u';	pStr++;
			*pStr	= 'e';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		case RTC_WEEK_WEDNESDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'W';	pStr++;
			*pStr	= 'e';	pStr++;
			*pStr	= 'd';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		case RTC_WEEK_THURSDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'T';	pStr++;
			*pStr	= 'h';	pStr++;
			*pStr	= 'u';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		case RTC_WEEK_FRIDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'F';	pStr++;
			*pStr	= 'r';	pStr++;
			*pStr	= 'i';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		case RTC_WEEK_SATURDAY:
			*pStr	= '(';	pStr++;
			*pStr	= 'S';	pStr++;
			*pStr	= 'a';	pStr++;
			*pStr	= 't';	pStr++;
			*pStr	= ')';	pStr++;
			break;

		default:
			break;
	}

	// space
	*pStr	= ' ';
	pStr++;

	// get hour data
	*pStr = STR_CHANGE_OFFSET + ((rtcDate.hour >> 4) & 0x03);
	pStr++;
	*pStr = STR_CHANGE_OFFSET + (rtcDate.hour & 0x0f);
	pStr++;
	*pStr = ':';
	pStr++;

	// get minute data
	*pStr = STR_CHANGE_OFFSET + ((rtcDate.minute >> 4) & 0x07);
	pStr++;
	*pStr = STR_CHANGE_OFFSET + (rtcDate.minute & 0x0f);
	pStr++;
	*pStr = ':';
	pStr++;

	// get second data
	*pStr = STR_CHANGE_OFFSET + ((rtcDate.second >> 4) & 0x07);
	pStr++;
	*pStr = STR_CHANGE_OFFSET + (rtcDate.second & 0x0f);
	pStr++;
	*pStr = ' ';
	pStr++;

	// display RTC count data
	printf(strBuff);

	printf("\n\n");
}


/*
 * set RTC time.
 * param	void
 * return	void
 * history	:	2010/04/05
 * 				2010/07/16 fix the reading method
 */
static void setRTC(void)
{
	RTC_DATE_INFO rtcDate;						// RTC count data
	unsigned char inputString[5] = {'0','x','\0','\0','\0'};
	unsigned char tempTime;
	short result;
	// Set RTC date
	// set year data
	printf("> Input BCD format.\n");
	printf("> Year (00 - 99) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.year		= tempTime;

	// set month data
	printf("> Month (01 - 12) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.month		= tempTime;

	// set day data
	printf("> Day (01 - 31) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.day		= tempTime;

    // set hour data
	printf("> Hour (00 - 23) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.hour = tempTime;

	// set minute data
	printf("> Minute (00 - 59) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.minute = tempTime;

	// set second data
	printf("> Second (00 - 59) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.second = tempTime;

	// set week data
	printf("> Week (0 - 6) :\n");
	fgets((char*)&inputString[PREFIX_LENGTH], sizeof(inputString) - PREFIX_LENGTH, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.week = tempTime;

	// RTC time set
	result = RTC_setTime(&rtcDate);
	if (result == RTC_INVALID_PARAMETER) {
		printf("input data is invalid\n");
	}
	printf("\n");
}


/*
 * count RTC interrupt.
 * param	void
 * return	void
 * history	:	2010/04/05
 */
static void countRTCInt(void)
{
	// display interrupt count value
	printf("> interrupt count value = %d\r\n", rtcIntCt);

}


/*
 * display menu
 * param	void
 * return	MAX menu No.
 * history	:	2010/04/05
 */
static unsigned char rtcMenuDisp(void)
{
	unsigned char	i = 0U;
	unsigned char	buff[RTC_MENU_MAX];


	// menu display
	while(rtcMainMenu[i].menuString != NULL) {
		sprintf(&buff[(i%2U)*37U], "%2d. %33s\n", i+1U, rtcMainMenu[i].menuString);

		if((i % 2U) == 0x01U) {
			printf(buff);
		}
		i++;
	}

	printf("\n\nPlease input number.\n> ");

	return	i;
}


/*
 * get menu number and check
 * param	MAX menu No.
 * return	input menu No.
 * history: 2010/04/05
 */
static unsigned char rtcMenuInput(unsigned char menuMax)
{
	unsigned char	inputString[5];										// user input data buffer
	unsigned char	inputNo;											// input menu number

	// input menu number
	fgets((char*)&inputString[0], sizeof(inputString), stdin);
	sscanf(inputString, "%d", (char*)&inputNo);

	// range check
	if((inputNo <= 0) || (inputNo > menuMax)) {
		inputNo = MENU_ERROR;
	}
	return	inputNo;
}


/*
 * dummy function
 * param: none
 * return: short									end status
 * history: 2010/04/05
 */
static void rtcDummy(void)
{
	return;
}


/*
 * RTC interrupt handler
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void rtcIntHandler(void)
{
	// call interrupt process function
	rtcInt();
}

/*
 * RTC interrupt process
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void rtcInt(void)
{
	// count up the interrupt counter
	rtcIntCt++;

	// reset interrupt flag
	RTC_resetIntFlag();
}

#endif // end of PE_RTC
