/**
 * @file	ct.c
 * @brief	clock timer driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_CT
#include <ct_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
void CT_resetTimer(void);
short CT_setTimerRun(unsigned char selectBit);
short CT_getCounterData(unsigned char *data, unsigned char number);
short CT_initInt(unsigned char intLevel);
short CT_controlInt(unsigned char selectBit);
short CT_resetIntFlag(unsigned char selectBit);
void CT_checkIntFlag(unsigned char *flag);


/**
 * @brief	reset a clock timer.
 * - 1. reset a clock timer.
 *
 * @return	none
 * @date	2009/12/25
 */
void CT_resetTimer(void)
{
	// clock timer reset
	RegModify(R08_CT_CTL, CTRST, (CTRST * CT_RESET));
}

/**
 * @brief	set a clock timer run / stop.
 * - 1. check parameters.
 * - 2. set a clock timer run/stop.
 *
 * @param	selectBit; unsigned char: 		select run / stop
 * -		CT_TIMER_STOP : clock timer stop
 * -		CT_TIMER_RUN  : clock timer run
 * @return	short:							end status
 * -		CT_SUCCESS : success
 * -		CT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short CT_setTimerRun(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CT_TIMER_RUN) {
		return	CT_INVALID_PARAMETER;
	}

	// clock timer run / stop
	RegModify(R08_CT_CTL, CTRUN, (selectBit << CTRUN_SFT));
	
	return	CT_SUCCESS;
}

/**
 * @brief	get a counter data.
 * - 1. get a counter data from register and set the data to "data".
 * 	-	Note: The correct counter value may not be read out (reading is unstable)
 * 			  if the counter register is read while counting is underway.
 * 	-		  a count value returns success with a continuation match twice
 * 			  and return a failure with a designation number not match.
 *
 * @param	*data; unsigned char:			counter data
 * -		CT_COUNTER_128 : 128Hz
 * -		CT_COUNTER_64  : 64Hz
 * -		CT_COUNTER_32  : 32Hz
 * -		CT_COUNTER_16  : 16Hz
 * -		CT_COUNTER_8   : 8Hz
 * -		CT_COUNTER_4   : 4Hz
 * -		CT_COUNTER_2   : 2Hz
 * -		CT_COUNTER_1   : 1Hz
 * @param	number; unsigned char:			number of a get count value
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		CT_SUCCESS : success
 * -		CT_FAILURE : get count value failure
 * @date	2009/12/25
 */
short CT_getCounterData(unsigned char *data, unsigned char number)
{
	unsigned char	i;														// counter
	unsigned char	cmp_data;												// compare data
	
	// counter data get
	for(i = 0; i < number; i++) {
		*data    = RegRead(R08_CT_CNT);										// data read
		cmp_data = RegRead(R08_CT_CNT);										// compare data read
		
		if(*data == cmp_data) {
			break;															// data OK
		}
	}
	
	// data read check
	if(i == number) {
		return	CT_FAILURE;
	}
	return	CT_SUCCESS;
}

/**
 * @brief	clock timer interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		CT_INT_LEVEL0 : level0
 * -		CT_INT_LEVEL1 : level1
 * -		CT_INT_LEVEL2 : level2
 * -		CT_INT_LEVEL3 : level3
 * -		CT_INT_LEVEL4 : level4
 * -		CT_INT_LEVEL5 : level5
 * -		CT_INT_LEVEL6 : level6
 * -		CT_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		CT_SUCCESS : success
 * -		CT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short CT_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= CT_INT_LEVELMAX) {
		return	CT_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(CT_INTLVL, intLevel);
	
	return	CT_SUCCESS;
}

/**
 * @brief	clock timer interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		CT_32INT_DISABLE : 32Hz signal interrupt disable
 * -		CT_32INT_ENABLE  : 32Hz signal interrupt enable
 * -		CT_8INT_DISABLE  :  8Hz signal interrupt disable
 * -		CT_8INT_ENABLE   :  8Hz signal interrupt enable
 * -		CT_2INT_DISABLE  :  2Hz signal interrupt disable
 * -		CT_2INT_ENABLE   :  2Hz signal interrupt enable
 * -		CT_1INT_DISABLE  :  1Hz signal interrupt disable
 * -		CT_1INT_ENABLE   :  1Hz signal interrupt enable
 * @return	short:							end status
 * -		CT_SUCCESS : success
 * -		CT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short CT_controlInt(unsigned char selectBit)
{
	// parameter check
	if((selectBit & ~CT_INT_MASK) != 0) {
		return	CT_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	CT_resetIntFlag(selectBit);
	
	// set interrupt enable
	RegWrite(R08_CT_IMSK, selectBit);
	
	return	CT_SUCCESS;
}

/**
 * @brief	clock timer interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	selectBit; unsigned char:		reset interrupt flag
 * -		CT_32INT_RESET : 32Hz signal interrupt flag reset
 * -		CT_8INT_RESET  :  8Hz signal interrupt flag reset
 * -		CT_2INT_RESET  :  2Hz signal interrupt flag reset
 * -		CT_1INT_RESET  :  1Hz signal interrupt flag reset
 * @return	short:							end status
 * -		CT_SUCCESS : success
 * -		CT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short CT_resetIntFlag(unsigned char selectBit)
{
	// parameter check
	if((selectBit & ~CT_INT_MASK) != 0) {
		return	CT_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegWrite(R08_CT_IFLG, selectBit);
	
	return	CT_SUCCESS;
}

/**
 * @brief	clock timer interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned char:			check interrupt flag
 * -		CT_32INT_NOTOCCURRED : 32Hz signal interrupt is not occurred
 * -		CT_32INT_OCCURRED    : 32Hz signal interrupt is occurred
 * -		CT_8INT_NOTOCCURRED  :  8Hz signal interrupt is not occurred
 * -		CT_8INT_OCCURRED     :  8Hz signal interrupt is occurred
 * -		CT_2INT_NOTOCCURRED  :  2Hz signal interrupt is not occurred
 * -		CT_2INT_OCCURRED     :  2Hz signal interrupt is occurred
 * -		CT_1INT_NOTOCCURRED  :  1Hz signal interrupt is not occurred
 * -		CT_1INT_OCCURRED     :  1Hz signal interrupt is occurred
 * @return	none
 * @date	2009/12/25
 */
void CT_checkIntFlag(unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R08_CT_IFLG);
}

#endif	// end of PE_CT
