/**
 * @file	lcd.c
 * @brief	LCD driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_LCD
#include <lcd_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static const unsigned char	DOT_DATA[] = {
	0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80
};


/*
 * Declarations of function prototype
 */
short LCD8_initPower(unsigned char selectMode, unsigned char selectVoltage);
short LCD8_init(unsigned char selectContrast, unsigned char selectDuty);
short LCD8_setSEGAssignment(unsigned char selectBit);
short LCD8_setCOMAssignment(unsigned char selectBit);
short LCD8_setDisplayArea(unsigned char selectBit);
short LCD8_setDisplayReverse(unsigned char selectBit);
short LCD8_controlDisplay(unsigned char selectBit);
short LCD8_setContrast(unsigned char selectBit);
short LCD8_display1Seg(unsigned char seg, unsigned char com, unsigned char data);
short LCD8_initInt(unsigned char intLevel);
short LCD8_controlInt(unsigned char selectBit);
void LCD8_resetIntFlag(void);
void LCD8_checkIntFlag(unsigned char *flag);


/**
 * @brief	LCD power initialize.
 * - 1. check parameters.
 * - 2. set a heavy load protection mode.
 * - 3. set a power source.
 *
 * @param	selectMode; unsigned char: 		select heavy load protection mode
 * -		LCD8_PROTECTMODE_OFF : heavy load protection mode off
 * -		LCD8_PROTECTMODE_ON  : heavy load protection mode on
 * @param	selectVoltage; unsigned char: 	select power source
 * -		LCD8_POWERSELECT_1V : power supply voltage 1.8-2.5V
 * -		LCD8_POWERSELECT_2V : power supply voltage 2.5-3.6V
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_initPower to LCD8_initPower.
 * @version
 * -		2009/12/25
 */
short LCD8_initPower(unsigned char selectMode, unsigned char selectVoltage)
{
	// parameter check
	if(	(selectMode > LCD8_PROTECTMODE_ON) ||
		(selectVoltage > LCD8_POWERSELECT_2V) ) {
			return	LCD8_INVALID_PARAMETER;
	}

	// LCD power initialize
	RegWrite(R08_LCD_VREG, ((selectMode << HVLD_SFT) | (selectVoltage << VCSEL_SFT)));

	return	LCD8_SUCCESS;
}

/**
 * @brief	LCD initialize.
 * - 1. check parameters.
 * - 2. set a LCD display off.
 * - 3. set a SEG assignment normal.
 * - 4. set a COM assignment normal.
 * - 5. set a display area 0.
 * - 6. set a display reverse normal.
 * - 7. set a LCD contrast adjustment.
 * - 8. set a LCD duty.
 * - 9. set a LCD display ram area all 0.
 *
 * @param	selectContrast; unsigned char: 	select contrast adjustment
 * -		LCD8_CONTRAST_0 : contrast 0/light
 * -		LCD8_CONTRAST_1 : contrast 1
 * -		LCD8_CONTRAST_2 : contrast 2
 * -		LCD8_CONTRAST_3 : contrast 3
 * -		LCD8_CONTRAST_4 : contrast 4
 * -		LCD8_CONTRAST_5 : contrast 5
 * -		LCD8_CONTRAST_6 : contrast 6
 * -		LCD8_CONTRAST_7 : contrast 7
 * -		LCD8_CONTRAST_8 : contrast 8
 * -		LCD8_CONTRAST_9 : contrast 9
 * -		LCD8_CONTRAST_A : contrast A
 * -		LCD8_CONTRAST_B : contrast B
 * -		LCD8_CONTRAST_C : contrast C
 * -		LCD8_CONTRAST_D : contrast D
 * -		LCD8_CONTRAST_E : contrast E
 * -		LCD8_CONTRAST_F : contrast F/dark
 * @param	selectDuty; unsigned char: 		select LCD duty
 * -		LCD8_DUTY_STATIC : static
 * -		LCD8_DUTY_2      : 1/2
 * -		LCD8_DUTY_3      : 1/3
 * -		LCD8_DUTY_4      : 1/4
 * -		LCD8_DUTY_8      : 1/8
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_init to LCD8_init.
 * @version
 * -		2009/12/25
 */
short LCD8_init(unsigned char selectContrast, unsigned char selectDuty)
{
	unsigned char	seg;

	// parameter check
	if(	(selectContrast >= LCD8_CONTRAST_MAX) ||
		(selectDuty >= LCD8_DUTY_MAX) ) {
			return	LCD8_INVALID_PARAMETER;
	}

	// LCD display off
	LCD8_controlDisplay(LCD8_DISPLAY_OFF);

	// SEG assignment set
	LCD8_setSEGAssignment(LCD8_SEGASIN_NORMAL);

	// COM assignment set
	LCD8_setCOMAssignment(LCD8_COMASIN_NORMAL);

	// display area set
	LCD8_setDisplayArea(LCD8_DISPLAY_AREA0);

	// display reverse set
	LCD8_setDisplayReverse(LCD8_DISPREV_NORMAL);

	// LCD contrast set
	LCD8_setContrast(selectContrast);

	// LCD duty set
	RegModify(R08_LCD_CCTL, LDUTY, (selectDuty << LDUTY_SFT));

	// LCD area all clear
	for(seg = 0; seg < LCD8_SEG_MAX; seg++) {
		RegWrite(R08_LCD_RAM+seg, 0x00);
	}

	return	LCD8_SUCCESS;
}

/**
 * @brief	set a segment output assignment.
 * - 1. check parameters.
 * - 2. set a segment output assignment.
 *
 * @param	selectBit; unsigned char: 		select segment output assignment
 * -		LCD8_SEGASIN_REVERSE : segment output assignment reverse
 * -		LCD8_SEGASIN_NORMAL  : segment output assignment normal
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_setSEGAssignment
 * 						 to LCD8_setSEGAssignment.
 * @version
 * -		2009/12/25
 */
short LCD8_setSEGAssignment(unsigned char selectBit)
{
	// parameter check
	if(selectBit > LCD8_SEGASIN_NORMAL) {
		return	LCD8_INVALID_PARAMETER;
	}

	// segment output assignment set
	RegModify(R08_LCD_DCTL, SEGREV, (selectBit << SEGREV_SFT));

	return	LCD8_SUCCESS;
}

/**
 * @brief	set a common output assignment.
 * - 1. check parameters.
 * - 2. set a common output assignment.
 *
 * @param	selectBit; unsigned char: 		select common output assignment
 * -		LCD8_COMASIN_REVERSE : common output assignment reverse
 * -		LCD8_COMASIN_NORMAL  : common output assignment normal
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_setCOMAssignment
 * 						 to LCD8_setCOMAssignment.
 * @version
 * -		2009/12/25
 */
short LCD8_setCOMAssignment(unsigned char selectBit)
{
	// parameter check
	if(selectBit > LCD8_COMASIN_NORMAL) {
		return	LCD8_INVALID_PARAMETER;
	}

	// common output assignment set
	RegModify(R08_LCD_DCTL, COMREV, (selectBit << COMREV_SFT));

	return	LCD8_SUCCESS;
}

/**
 * @brief	set a LCD display area.
 * - 1. check parameters.
 * - 2. set a LCD display area.
 *
 * @param	selectBit; unsigned char: 		select display area
 * -		LCD8_DISPLAY_AREA0 : display area 0
 * -		LCD8_DISPLAY_AREA1 : display area 1
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_setDisplayArea
 * 						 to LCD8_setDisplayArea.
 * @version
 * -		2009/12/25
 */
short LCD8_setDisplayArea(unsigned char selectBit)
{
	// parameter check
	if(selectBit > LCD8_DISPLAY_AREA1) {
		return	LCD8_INVALID_PARAMETER;
	}

	// LCD display area set
	RegModify(R08_LCD_DCTL, DSPAR, (selectBit << DSPAR_SFT));

	return	LCD8_SUCCESS;
}

/**
 * @brief	set a LCD display reverse.
 * - 1. check parameters.
 * - 2. set a LCD display reverse.
 *
 * @param	selectBit; unsigned char: 		select display reverse
 * -		LCD_DISPREV_REVERSE : display reverse
 * -		LCD_DISPREV_NORMAL  : display normal
 * @return	short:							end status
 * -		LCD_SUCCESS : success
 * -		LCD_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_setDisplayReverse
 * 						 to LCD8_setDisplayReverse.
 * @version
 * -		2009/12/25
 */
short LCD8_setDisplayReverse(unsigned char selectBit)
{
	// parameter check
	if(selectBit > LCD8_DISPREV_NORMAL) {
		return	LCD8_INVALID_PARAMETER;
	}

	// LCD display reverse set
	RegModify(R08_LCD_DCTL, DSPREV, (selectBit << DSPREV_SFT));

	return	LCD8_SUCCESS;
}

/**
 * @brief	control a LCD display.
 * - 1. check parameters.
 * - 2. set a LCD display control.
 *
 * @param	selectBit; unsigned char: 		select LCD display control
 * -		LCD8_DISPLAY_OFF    : LCD display off
 * -		LCD8_DISPLAY_NORMAL : LCD display normal
 * -		LCD8_DISPLAY_ALLON  : LCD display all on(dynamic)
 * -		LCD8_DISPLAY_ALLOFF : LCD display all off(static)
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_controlDisplay
 * 						 to LCD8_controlDisplay.
 * @version
 * -		2009/12/25
 */
short LCD8_controlDisplay(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= LCD8_DISPLAY_MAX) {
		return	LCD8_INVALID_PARAMETER;
	}

	// LCD display control
	RegModify(R08_LCD_DCTL, DSPC, (selectBit << DSPC_SFT));

	return	LCD8_SUCCESS;
}

/**
 * @brief	set a LCD contrast.
 * - 1. check parameters.
 * - 2. set a LCD contrast adjustment.
 *
 * @param	selectBit; unsigned char: 		select contrast adjustment
 * -		LCD8_CONTRAST_0 : contrast 0/light
 * -		LCD8_CONTRAST_1 : contrast 1
 * -		LCD8_CONTRAST_2 : contrast 2
 * -		LCD8_CONTRAST_3 : contrast 3
 * -		LCD8_CONTRAST_4 : contrast 4
 * -		LCD8_CONTRAST_5 : contrast 5
 * -		LCD8_CONTRAST_6 : contrast 6
 * -		LCD8_CONTRAST_7 : contrast 7
 * -		LCD8_CONTRAST_8 : contrast 8
 * -		LCD8_CONTRAST_9 : contrast 9
 * -		LCD8_CONTRAST_A : contrast A
 * -		LCD8_CONTRAST_B : contrast B
 * -		LCD8_CONTRAST_C : contrast C
 * -		LCD8_CONTRAST_D : contrast D
 * -		LCD8_CONTRAST_E : contrast E
 * -		LCD8_CONTRAST_F : contrast F/dark
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_setContrast to LCD8_setContrast.
 * @version
 * -		2009/12/25
 */
short LCD8_setContrast(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= LCD8_CONTRAST_MAX) {
		return	LCD8_INVALID_PARAMETER;
	}

	// LCD contrast adjustment set
	RegWrite(R08_LCD_CADJ, (selectBit << LC_SFT));

	return	LCD8_SUCCESS;
}

/**
 * @brief	display 1-segment.
 * - 1. check parameters.
 * - 2. turn-off/turn-on 1-segment of designation.
 *
 * @param	seg; unsigned char: 			select display segment number
 * -		LCD8_SEG00 - LCD8_SEG39 : SEG00 - SEG39
 * @param	com; unsigned char: 			select display common number
 * -		LCD8_COM0 : COM0
 * -		LCD8_COM1 : COM1
 * -		LCD8_COM2 : COM2
 * -		LCD8_COM3 : COM3
 * -		LCD8_COM4 : COM4
 * -		LCD8_COM5 : COM5
 * -		LCD8_COM6 : COM6
 * -		LCD8_COM7 : COM7
 * @param	data; unsigned char: 			select display data
 * -		LCD8_DOT_L : data turn-off
 * -		LCD8_DOT_H : data turn-on
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_display1Seg
 * 						 to LCD8_display1Seg.
 * @version
 * -		2009/12/25
 */
short LCD8_display1Seg(unsigned char seg, unsigned char com, unsigned char data)
{
	// parameter check
	if(	(seg >= LCD8_SEG_MAX ) ||
		(com >= LCD8_COM_MAX ) ||
		(data > LCD8_DOT_H ) ) {
			return	LCD8_INVALID_PARAMETER;
	}

	// 1-dot display
	RegModify(R08_LCD_RAM+seg, DOT_DATA[com], (data * DOT_DATA[com]));

	return	LCD8_SUCCESS;
}

/**
 * @brief	LCD interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		LCD8_INT_LEVEL0 : level0
 * -		LCD8_INT_LEVEL1 : level1
 * -		LCD8_INT_LEVEL2 : level2
 * -		LCD8_INT_LEVEL3 : level3
 * -		LCD8_INT_LEVEL4 : level4
 * -		LCD8_INT_LEVEL5 : level5
 * -		LCD8_INT_LEVEL6 : level6
 * -		LCD8_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_initInt to LCD8_initInt
 * @version
 * -		2009/12/25

 */
short LCD8_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= LCD8_INT_LEVELMAX) {
		return	LCD8_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(LCD_INTLVL, intLevel);

	return	LCD8_SUCCESS;
}

/**
 * @brief	LCD interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		LCD8_INT_DISABLE : interrupt disable
 * -		LCD8_INT_ENABLE  : interrupt enable
 * @return	short:							end status
 * -		LCD8_SUCCESS : success
 * -		LCD8_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from LCD_controlInt to LCD8_controlInt.
 * @version
 * -		2009/12/25
 */
short LCD8_controlInt(unsigned char selectBit)
{
	// parameter check
	if(selectBit > LCD8_INT_ENABLE) {
		return	LCD8_INVALID_PARAMETER;
	}

	// reset interrupt flag
	LCD8_resetIntFlag();

	// set interrupt enable / disable
	RegWrite(R08_LCD_IMSK, selectBit);

	return	LCD8_SUCCESS;
}

/**
 * @brief	LCD interrupt flag reset.
 * - 1. reset an interrupt status.
 *
 * @return	none
 * @date	2010/04/05 : Rename the function from LCD_resetIntFlag
 * 						 to LCD8_resetIntFlag.
 * @version
 * -		2009/12/25
 */
void LCD8_resetIntFlag(void)
{
	// reset interrupt flag
	RegWrite(R08_LCD_IFLG, LCD8_INT_RESET);
}

/**
 * @brief	LCD interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 *
 * @param	*flag; unsigned char:			check interrupt flag
 * -		LCD8_INT_NOTOCCURRED : frame interrupt is not occurred
 * -		LCD8_INT_OCCURRED    : frame interrupt is occurred
 * @return	none
 * @date	2010/04/05 : Rename the function from LCD_checkIntFlag
 * 						 to LCD8_checkIntFlag.
 * @version
 * -		2009/12/25
 */
void LCD8_checkIntFlag(unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R08_LCD_IFLG);
}

#endif	// end of PE_LCD
