/**
 * @file	osc.c
 * @brief	osc driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 */

#include <s1c176xx.h>

#include <osc_api.h>
#include <osc.h>

#ifdef	PE_CLG_OSC

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static const unsigned char	selectSysClk[] = {
	IOSCEN,																	// select IOSC
	OSC1EN,																	// select OSC1
	OSC3EN,																	// select OSC3
	OSC1EN																	// select OSC1
};

/*
 * Declarations of function prototype
 */
short CLG_OSC_setClockSource(unsigned char selectHSCLK, unsigned char selectCLKSRC);
short CLG_OSC_setWaitCycle(unsigned char selectIOSC, unsigned char selectOSC3);
short CLG_OSC_controlOscillation(unsigned char selectBit);
short CLG_OSC_setNoiseFilter(unsigned char selectBit);
short CLG_OSC_setLCDClock(unsigned char selectOsc, unsigned char selectRatio);
short CLG_OSC_controlLCDClock(unsigned char selectBit);
short CLG_OSC_setFOUTHDivision(unsigned char selectBit);
short CLG_OSC_controlFOUT(unsigned char selectBit);
short CLG_OSC_setT8OSC1Division(unsigned char selectBit);
short CLG_OSC_controlT8OSC1(unsigned char selectBit);
short CLG_OSC_setSVDClock(unsigned char selectBit);
short CLG_OSC_controlSVD(unsigned char selectBit);
short CLG_OSC_setRFCClock(unsigned char selectOsc, unsigned char selectRatio);
short CLG_OSC_controlRFC(unsigned char selectBit);
#if	defined(S1C17624) || defined(S1C17604)
short CLG_OSC_setT16A2Clock(unsigned char chNo, CLG_OSC_SET_T16A2 *select);
short CLG_OSC_controlT16A2(unsigned char chNo, unsigned char selectBit);
short CLG_OSC_controlRTC(unsigned char selectBit);
#endif	// end of S1C17624/S1C17604

/**
 * @brief	set a clock source.
 * - 1. check parameters.
 * - 2. set a selected clock.
 *
 * @param	selectHSCLK; unsigned char: 	select High-speed clock
 * -		CLG_OSC_SELHSCLK_IOSC : IOSC
 * -		CLG_OSC_SELHSCLK_OSC3 : OSC3
 * @param	selectCLKSRC; unsigned char: 	select system clock source
 * -		CLG_OSC_SELCLKSRC_HSCLK : HSCLK
 * -		CLG_OSC_SELCLKSRC_OSC1  : OSC1
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the  function from OSC_setClockSource
 * 						 to CLG_OSC_setClockSource.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setClockSource(unsigned char selectHSCLK, unsigned char selectCLKSRC)
{
	// parameter check
	if(	(selectHSCLK > CLG_OSC_SELHSCLK_OSC3) ||
		(selectCLKSRC > CLG_OSC_SELCLKSRC_OSC1) ) {
			return	CLG_OSC_INVALID_PARAMETER;
	}

	// set clock source
	RegWrite(R08_OSC_SRC, ((selectHSCLK << HSCLKSEL_SFT) | (selectCLKSRC << R08_OSC_SRC_CLKSRC_SFT)));

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	set an oscillation stable wait cycle.
 * - 1. check parameters.
 * - 2. set a selected stable wait cycle.
 *
 * @param	selectIOSC; unsigned char: 		select IOSC wait cycle
 * -		CLG_OSC_SELIOSC_64 : 64 cycle
 * -		CLG_OSC_SELIOSC_32 : 32 cycle
 * -		CLG_OSC_SELIOSC_16 : 16 cycle
 * -		CLG_OSC_SELIOSC_8  :  8 cycle
 * @param	selectOSC3; unsigned char: 		select OSC3 wait cycle
 * -		CLG_OSC_SELOSC3_1024 : 1024 cycle
 * -		CLG_OSC_SELOSC3_512  :  512 cycle
 * -		CLG_OSC_SELOSC3_256  :  256 cycle
 * -		CLG_OSC_SELOSC3_128  :  128 cycle
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setWaitCycle
 * 						 to CLG_OSC_setWaitCycle.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setWaitCycle(unsigned char selectIOSC, unsigned char selectOSC3)
{
	// parameter check
	if(	(selectIOSC >= CLG_OSC_SELIOSC_MAX) ||
		(selectOSC3 >= CLG_OSC_SELOSC3_MAX) ) {
			return	CLG_OSC_INVALID_PARAMETER;
	}

	// set oscillation stable waiting cycle
	RegModify(R08_OSC_CTL, (IOSCWT | OSC3WT), ((selectIOSC << IOSCWT_SFT) | (selectOSC3 << OSC3WT_SFT)));

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	oscillation enable / disable.
 * - 1. check parameters.
 * - 2. set an oscillation enable/disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		CLG_OSC_IOSC_DISABLE : IOSC oscillation disable
 * -		CLG_OSC_IOSC_ENABLE  : IOSC oscillation enable
 * -		CLG_OSC_OSC1_DISABLE : OSC1 oscillation disable
 * -		CLG_OSC_OSC1_ENABLE  : OSC1 oscillation enable
 * -		CLG_OSC_OSC3_DISABLE : OSC3 oscillation disable
 * -		CLG_OSC_OSC3_ENABLE  : OSC3 oscillation enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_FAILURE : oscillation stop failure
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_controlOscillation
 * 						 to CLG_OSC_controlOscillation.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_controlOscillation(unsigned char selectBit)
{
	// parameter check
	if(selectBit > OSCEN_MSK) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// system clock check
	if((selectBit & selectSysClk[(RegRead(R08_OSC_SRC) & (HSCLKSEL | R08_OSC_SRC_CLKSRC))]) == OSC_DISABLE ) {
		return	CLG_OSC_FAILURE;
	}

	// oscillation enable / disable
	RegModify(R08_OSC_CTL, OSCEN_MSK, selectBit);

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	noise filter enable / disable.
 * - 1. check parameters.
 * - 2. set a noise filter enable/disable.
 *
 * @param	selectBit; unsigned char: 		select noise filter enable / disable
 * -		CLG_OSC_NOISE_RESET_DISABLE : noise filter disable for RESET input
 * -		CLG_OSC_NOISE_RESET_ENABLE  : noise filter enable for RESET input
 * -		CLG_OSC_NOISE_NMI_DISABLE   : noise filter disable for NMI input
 * -		CLG_OSC_NOISE_NMI_ENABLE    : noise filter enable for NMI input
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setNoiseFilter
 * 						 to CLG_OSC_setNoiseFilter.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setNoiseFilter(unsigned char selectBit)
{
	// parameter check
	if(selectBit > NFEN_MSK) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// noise filter enable / disable
	RegWrite(R08_OSC_NFEN, selectBit);

	return	CLG_OSC_SUCCESS;
}

#endif	// end of PE_CLG_OSC


#ifdef	PE_LCD
/**
 * @brief	set a LCD clock.
 * - 1. check parameters.
 * - 2. disable LCD clock.
 * - 3. set a LCD clock source.
 * - 4. set a division ratio, if a LCD clock source is HSCLK.
 * - 5. enable LCD clock.
 *
 * @param	selectOsc; unsigned char: 		select LCD clock source
 * -		CLG_OSC_LCDCLK_HSCLK : HSCLK
 * -		CLG_OSC_LCDCLK_OSC1  : OSC1
 * @param	selectRatio; unsigned char: 	select LCD clock division ratio, if a LCD clock source is HSCLK
 * -		CLG_OSC_LCDRATIO_32  : HSCLK 1/32
 * -		CLG_OSC_LCDRATIO_64  : HSCLK 1/64
 * -		CLG_OSC_LCDRATIO_128 : HSCLK 1/128
 * -		CLG_OSC_LCDRATIO_256 : HSCLK 1/256
 * -		CLG_OSC_LCDRATIO_512 : HSCLK 1/512
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setLCDClock
 * 						 to CLG_OSC_setLCDClock.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setLCDClock(unsigned char selectOsc, unsigned char selectRatio)
{
	// parameter check
	if(	(selectOsc > CLG_OSC_LCDCLK_OSC1) ||
		(selectRatio >= CLG_OSC_LCDRATIO_MAX) ) {
			return	CLG_OSC_INVALID_PARAMETER;
	}

	// LCD clock disable
	CLG_OSC_controlLCDClock(CLG_OSC_LCDCLK_DISABLE);

	// LCD clock source set
	RegModify(R08_OSC_LCLK, LCKSRC, (selectOsc << LCKSRC_SFT));

	// LCD clock division ratio set for HSCLK
	if(selectOsc == CLG_OSC_LCDCLK_HSCLK) {
		RegModify(R08_OSC_LCLK, LCKDV, (selectRatio << LCKDV_SFT));
	}

	// LCD clock enable
	CLG_OSC_controlLCDClock(CLG_OSC_LCDCLK_ENABLE);

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	LCD clock enable / disable.
 * - 1. check parameters.
 * - 2. set a LCD clock enable/disable.
 *
 * @param	selectBit; unsigned char: 		select enable / disable
 * -		CLG_OSC_LCDCLK_DISABLE : LCD clock disable
 * -		CLG_OSC_LCDCLK_ENABLE  : LCD clock enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_controlLCDClock
 * 						 to CLG_OSC_controlLCDClock.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_controlLCDClock(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CLG_OSC_LCDCLK_ENABLE) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// LCD clock enable / disable
	RegModify(R08_OSC_LCLK, LCKEN, (selectBit << LCDEN_SFT));

	return	CLG_OSC_SUCCESS;
}

#endif	// end of PE_LCD


#ifdef	PE_CLG_OSC
/**
 * @brief	set a FOUTH clock division ratio.
 * - 1. check parameters.
 * - 2. set a FOUTH clock division ratio.
 *
 * @param	selectBit; unsigned char: 		select FOUTH clock division ratio
 * -		CLG_OSC_FOUTCLK_1 : HSCLK 1/1
 * -		CLG_OSC_FOUTCLK_2 : HSCLK 1/2
 * -		CLG_OSC_FOUTCLK_4 : HSCLK 1/4
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setFOUTHDivision
 * 						 to CLG_OSC_setFOUTHDivision.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setFOUTHDivision(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= CLG_OSC_FOUTCLK_MAX) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// FOUT clock output disable
	CLG_OSC_controlFOUT((RegRead(R08_OSC_FOUT) & FOUT1E ) | CLG_OSC_FOUTH_DISABLE);

	// set FOUTH clock division ratio
	RegModify(R08_OSC_FOUT, FOUTHD, (selectBit << FOUTHD_SFT));

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	FOUT clock output enable / disable.
 * - 1. check parameters.
 * - 2. set a FOUT clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select FOUT clock output enable / disable
 * -		CLG_OSC_FOUTH_DISABLE : FOUTH clock output disable
 * -		CLG_OSC_FOUTH_ENABLE  : FOUTH clock output enable
 * -		CLG_OSC_FOUT1_DISABLE : FOUT1 clock output disable
 * -		CLG_OSC_FOUT1_ENABLE  : FOUT1 clock output enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_controlFOUT
 * 						 to CLG_OSC_controlFOUT.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_controlFOUT(unsigned char selectBit)
{
	// parameter check
	if(selectBit > FOUT_MSK) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// FOUT clock enable / disable
	RegModify(R08_OSC_FOUT, FOUT_MSK, selectBit);

	return	CLG_OSC_SUCCESS;
}

#endif	// end of PE_CLG_OSC


#ifdef	PE_T8OSC1
/**
 * @brief	set a T8OSC1 clock division ratio.
 * - 1. check parameters.
 * - 2. set a T8OSC1 clock division ratio.
 *
 * @param	selectBit; unsigned char: 		select T8OSC1 clock division ratio
 * -		CLG_OSC_T8OSC1CLK_1  : HSCLK 1/1
 * -		CLG_OSC_T8OSC1CLK_2  : HSCLK 1/2
 * -		CLG_OSC_T8OSC1CLK_4  : HSCLK 1/4
 * -		CLG_OSC_T8OSC1CLK_8  : HSCLK 1/8
 * -		CLG_OSC_T8OSC1CLK_16 : HSCLK 1/16
 * -		CLG_OSC_T8OSC1CLK_32 : HSCLK 1/32
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setT8OSCDivision
 * 						 to CLG_OSC_setT8OSCDivision.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setT8OSC1Division(unsigned char selectBit)
{
	// parameter check
	if(selectBit >= CLG_OSC_T8OSC1CLK_MAX) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// T8OSC1 clock output disable
	CLG_OSC_controlT8OSC1(CLG_OSC_T8OSC1_DISABLE);

	// set T8OSC1 clock division ratio
	RegModify(R08_OSC_T8OSC1, T8O1CK, (selectBit << T8O1CK_SFT));

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	T8OSC1 clock output enable / disable.
 * - 1. check parameters.
 * - 2. set a T8OSC1 clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select T8OSC1 clock output enable / disable
 * -		CLG_OSC_T8OSC1_DISABLE : T8OSC1 clock output disable
 * -		CLG_OSC_T8OSC1_ENABLE  : T8OSC1 clock output enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_controlT8OSC1
 * 						 to CLG_OSC_controlT8OSC1.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_controlT8OSC1(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CLG_OSC_T8OSC1_ENABLE) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// T8OSC1 clock enable / disable
	RegModify(R08_OSC_T8OSC1, T8O1CE, (selectBit << T8O1CE_SFT));

	return	CLG_OSC_SUCCESS;
}

#endif	// end of PE_T8OSC1


#ifdef	PE_SVD
/**
 * @brief	set a SVD clock.
 * - 1. check parameters.
 * - 2. set a SVD clock source.
 *
 * @param	selectBit; unsigned char: 		select SVD clock source
 * -		CLG_OSC_SVDCLK_HSCLK : HSCLK 1/512
 * -		CLG_OSC_SVDCLK_OSC1  : OSC1
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setSVDClock
 * 						 to CLG_OSC_setSVDClock.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setSVDClock(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CLG_OSC_SVDCLK_OSC1) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// SVD clock disable
	CLG_OSC_controlSVD(CLG_OSC_SVD_DISABLE);

	// set SVD clock source
	RegModify(R08_OSC_SVD, SVDSRC, (selectBit << SVDSRC_SFT));

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	SVD clock output enable / disable.
 * - 1. check parameters.
 * - 2. set a SVD clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select SVD clock output enable / disable
 * -		CLG_OSC_SVD_DISABLE : SVD clock output disable
 * -		CLG_OSC_SVD_ENABLE  : SVD clock output enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_controlSVD
 * 						 to CLG_OSC_controlSVD.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_controlSVD(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CLG_OSC_SVD_ENABLE) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// SVD clock enable / disable
	RegModify(R08_OSC_SVD, SVDCKEN, (selectBit << SVDCKEN_SFT));

	return	CLG_OSC_SUCCESS;
}

#endif	// end of PE_SVD


#ifdef	PE_RFC
/**
 * @brief	set a RFC clock.
 * - 1. check parameters.
 * - 2. disable RFC clock.
 * - 3. set a RFC clock source.
 * - 4. set a division ratio, if a RFC clock source is HSCLK.
 * - 5. enable RFC clock.
 *
 * @param	selectOsc; unsigned char: 		select RFC clock source
 * -		CLG_OSC_RFCCLK_HSCLK : HSCLK
 * -		CLG_OSC_RFCCLK_OSC1  : OSC1
 * @param	selectRatio; unsigned char: 	select RFC clock division ratio, if a RFC clock source is HSCLK
 * -		CLG_OSC_RFCRATIO_1 : HSCLK 1/1
 * -		CLG_OSC_RFCRATIO_2 : HSCLK 1/2
 * -		CLG_OSC_RFCRATIO_4 : HSCLK 1/4
 * -		CLG_OSC_RFCRATIO_8 : HSCLK 1/8
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_setRFCClock
 * 						 to CLG_OSC_setRFCClock.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_setRFCClock(unsigned char selectOsc, unsigned char selectRatio)
{
	// parameter check
	if(	(selectOsc > CLG_OSC_RFCCLK_OSC1) ||
		(selectRatio >= CLG_OSC_RFCRATIO_MAX) ) {
			return	CLG_OSC_INVALID_PARAMETER;
	}

	// RFC clock disable
	CLG_OSC_controlRFC(CLG_OSC_RFCCLK_DISABLE);

	// RFC clock source set
	RegModify(R08_OSC_RFC, RFTCKSRC, (selectOsc << RFTCKSRC_SFT));

	// RFC clock division ratio set for HSCLK
	if(selectOsc == CLG_OSC_RFCCLK_HSCLK) {
		RegModify(R08_OSC_RFC, RFTCKDV, (selectRatio << RFTCKDV_SFT));
	}

	// RFC clock enable
	CLG_OSC_controlRFC(CLG_OSC_RFCCLK_ENABLE);

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	RFC clock enable / disable.
 * - 1. check parameters.
 * - 2. set a RFC clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select RFC clock output enable / disable
 * -		CLG_OSC_RFCCLK_DISABLE : RFC clock output disable
 * -		CLG_OSC_RFCCLK_ENABLE  : RFC clock output enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05 : Rename the function from OSC_controlRFC
 * 						 to CLG_OSC_controlRFC.
 * @version
 * -		2009/12/25
 */
short CLG_OSC_controlRFC(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CLG_OSC_RFCCLK_ENABLE) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// RFC clock enable / disable
	RegModify(R08_OSC_RFC, RFTCKEN, (selectBit << RFTCKEN_SFT));

	return	CLG_OSC_SUCCESS;
}

#endif	// end of PE_RFC


#ifdef	PE_T16A2
/**
 * @brief	set a timer input clock.
 * - 1. check parameters.
 * - 2. stop an PWM timer.
 * - 3. set a count clock.
 * - 4. set a clock source.
 * - 5. set a multi-comparator/capture mode.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		CLG_OSC_T16A2_CH0 : channel 0
 * -		CLG_OSC_T16A2_CH1 : channel 1
 * @param	*select; OSC_SET_T16A2: 		select timer input clock
 * -		clkdiv; unsigned short:			select count clock
 * 	-			CLG_OSC_T16A2RATIO_1     : clock source 1/1
 * 	-			CLG_OSC_T16A2RATIO_2     : clock source 1/2
 * 	-			CLG_OSC_T16A2RATIO_4     : clock source 1/4
 * 	-			CLG_OSC_T16A2RATIO_8     : clock source 1/8
 * 	-			CLG_OSC_T16A2RATIO_16    : clock source 1/16
 * 	-			CLG_OSC_T16A2RATIO_32    : clock source 1/32
 * 	-			CLG_OSC_T16A2RATIO_64    : clock source 1/64
 * 	-			CLG_OSC_T16A2RATIO_128   : clock source 1/128
 * 	-			CLG_OSC_T16A2RATIO_256   : clock source 1/256
 * 	-			CLG_OSC_T16A2RATIO_512   : clock source 1/512
 * 	-			CLG_OSC_T16A2RATIO_1024  : clock source 1/1024
 * 	-			CLG_OSC_T16A2RATIO_2048  : clock source 1/2048
 * 	-			CLG_OSC_T16A2RATIO_4096  : clock source 1/4096
 * 	-			CLG_OSC_T16A2RATIO_8192  : clock source 1/8192
 * 	-			CLG_OSC_T16A2RATIO_16384 : clock source 1/16384
 * -		clksrc; unsigned short:			select count clock source
 * 	-			CLG_OSC_T16A2CLK_IOSC : IOSC
 * 	-			CLG_OSC_T16A2CLK_OSC1 : OSC1
 * 	-			CLG_OSC_T16A2CLK_OSC3 : OSC3
 * 	-			CLG_OSC_T16A2CLK_EXCL : external clock
 * -		multimd; unsigned short:		select timer mode
 * 	-			CLG_OSC_T16A2MD_NORMAL : normal channel mode
 * 	-			CLG_OSC_T16A2MD_MULTI  : multi-comparator/capture mode
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05
 */
short CLG_OSC_setT16A2Clock(unsigned char chNo, CLG_OSC_SET_T16A2 *select)
{
	// parameter check
	if(	(chNo >= CLG_OSC_T16A2_CH_MAX) ||
		(select->clkdiv >= CLG_OSC_T16A2RATIO_MAX) ||
		(select->clksrc >= CLG_OSC_T16A2CLK_MAX) ||
		(select->multimd > CLG_OSC_T16A2MD_MULTI) ) {
			return	CLG_OSC_INVALID_PARAMETER;
	}


	// set input clock(clock division, clock source, multi mode)
	RegWrite(R08_T16A_CLK(chNo),
		((select->clkdiv << CLKDIV_SFT) | (select->clksrc << R08_T16A_CLK_CLKSRC_SFT) | (select->multimd << MULTIMD_SFT)));

	return	CLG_OSC_SUCCESS;
}

/**
 * @brief	T16A2 clock enable / disable.
 * - 1. check parameters.
 * - 2. set aT16A2 clock output enable/disable.
 *
 * @param	chNo; unsigned char:			channel number
 * -		CLG_OSC_T16A2_CH0 : channel 0
 * -		CLG_OSC_T16A2_CH1 : channel 1
 * @param	selectBit; unsigned char: 		select T16A2 clock output enable / disable
 * -		CLG_OSC_T16A2CLK_DISABLE : T16A2 clock output disable
 * -		CLG_OSC_T16A2CLK_ENABLE  : T16A2 clock output enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05
 */

short CLG_OSC_controlT16A2(unsigned char chNo, unsigned char selectBit)
{
	// parameter check
	if(	(chNo >= CLG_OSC_T16A2_CH_MAX) ||
		(selectBit > CLG_OSC_T16A2CLK_ENABLE) ) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// T16A2 clock enable / disable
	RegModify(R08_T16A_CLK(chNo), CLKEN, (selectBit << CLKEN_SFT));

	return	CLG_OSC_SUCCESS;
}

#endif // end of PE_T16A2


#ifdef PE_RTC
/**
 * @brief	RTC clock enable / disable.
 * - 1. check parameters.
 * - 2. set a RTC clock output enable/disable.
 *
 * @param	selectBit; unsigned char: 		select RTC clock output enable / disable
 * -		CLG_OSC_RTCCLK_DISABLE : RTC clock output disable
 * -		CLG_OSC_RTCCLK_ENABLE  : RTC clock output enable
 * @return	short:							end status
 * -		CLG_OSC_SUCCESS : success
 * -		CLG_OSC_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/05
 */
short CLG_OSC_controlRTC(unsigned char selectBit)
{
	// parameter check
	if(selectBit > CLG_OSC_RTCCLK_ENABLE) {
		return	CLG_OSC_INVALID_PARAMETER;
	}

	// RFC clock enable / disable
	RegModify(R08_RTC_CC, RTCCE, (selectBit << RTCCE_SFT));

	return	CLG_OSC_SUCCESS;
}
#endif // end of PE_RTC
