/**
 * @file	spi.c
 * @brief	SPI driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_SPI
#include <spi_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static volatile unsigned char	*SPI_INTLVL_REG[] = {
	SPI_INTLVL_CH0															// Ch. 0
};


/*
 * Declarations of function prototype
 */
short SPI_init(unsigned char chNo, SPI_INIT_PARAM *param);
short SPI_setTransmitData(unsigned char chNo, unsigned char data);
short SPI_getReceiveData(unsigned char chNo, unsigned char *data);
short SPI_setComEnable(unsigned char chNo, unsigned short selectBit);
short SPI_initInt(unsigned char chNo, unsigned char intLevel);
short SPI_controlInt(unsigned char chNo, unsigned short selectBit);
short SPI_checkIntFlag(unsigned char chNo, unsigned short *flag);
short SPI_checkBusyFlag(unsigned char chNo, unsigned short *flag);


/**
 * @brief	SPI initialize.
 * - 1. check parameters.
 * - 2. set a SPI disable.
 * - 3. set SPI ports.
 * - 4. set a clock source.
 * - 5. set a LSB/MSB first.
 * - 6. set a clock phase.
 * - 7. set a clock polarity.
 * - 8. set a master/slave mode.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		SPI_CH0 : channel 0
 * @param	*param; SPI_INIT_PARAM:			SPI initialize parameter
 * -		selectClk; unsigned short:		select clock source
 * 	-			SPI_CLK_PCLK : PCLK 1/4
 * 	-			SPI_CLK_T16  : 16-bit timer Ch.1
 * -		selectMLSB; unsigned short:		select LSB/MSB first
 * 	-			SPI_MSB_FIRST : MSB first
 * 	-			SPI_LSB_FIRST : LSB first
 * -		selectPhase; unsigned short:	select clock phase
 * 	-			SPI_PHASE_DATAIN  : clock phase data in
 * 	-			SPI_PHASE_DATAOUT : clock phase data out
 * -		selectPolarity; unsigned short:	select clock polarity
 * 	-			SPI_ACTIVE_H : active high
 * 	-			SPI_ACTIVE_L : active low
 * -		selectMode; unsigned short:		select master/slave mode
 * 	-			SPI_MODE_SLAVE  : SPI slave mode
 * 	-			SPI_MODE_MASTER : SPI master mode
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_init(unsigned char chNo, SPI_INIT_PARAM *param)
{
	// parameter check
	if(	(chNo >= SPI_CH_MAX) ||
		(param->selectClk > SPI_CLK_T16) ||
		(param->selectMLSB > SPI_LSB_FIRST) ||
		(param->selectPhase > SPI_PHASE_DATAOUT) ||
		(param->selectPolarity > SPI_ACTIVE_L) ||
		(param->selectMode > SPI_MODE_MASTER) ) {
			return	SPI_INVALID_PARAMETER;
	}

	// SPI disable
	SPI_setComEnable(chNo, SPI_CTL_DISABLE);

	// set SPI port
	MUX_setSPIport(chNo, MUX_ON);

	// set initialize parameter
	RegModify(R16_SPI_CTL(chNo), SPI_INIT_MASK,
		((param->selectClk << MCLK_SFT) | (param->selectMLSB << MLSB_SFT) | (param->selectPhase << CPHA_SFT) |
		(param->selectPolarity << CPOL_SFT) | (param->selectMode << MSSL_SFT)));

	return	SPI_SUCCESS;
}

/**
 * @brief	set a transmit data.
 * - 1. check parameters.
 * - 2. set a transmit data.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		SPI_CH0 : channel 0
 * @param	data; unsigned char:			transmit data
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_setTransmitData(unsigned char chNo, unsigned char data)
{
	// parameter check
	if(chNo >= SPI_CH_MAX) {
		return	SPI_INVALID_PARAMETER;
	}

	// set transmit data
	RegWrite(R16_SPI_TXD(chNo), (unsigned short)data);

	return	SPI_SUCCESS;
}

/**
 * @brief	get a receive data.
 * - 1. check parameters.
 * - 2. get a receive data and set to "data".
 *
 * @param	chNo; unsigned char: 			channel number
 * -		SPI_CH0 : channel 0
 * @param	*data; unsigned char:			receive data
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_getReceiveData(unsigned char chNo, unsigned char *data)
{
	// parameter check
	if(chNo >= SPI_CH_MAX) {
		return	SPI_INVALID_PARAMETER;
	}

	// get receive data
	*data = (unsigned char)RegRead(R16_SPI_RXD(chNo));

	return	SPI_SUCCESS;
}

/**
 * @brief	set a SPI enable / disable.
 * - 1. check parameters.
 * - 2. set a SPI enable/disable.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		SPI_CH0 : channel 0
 * @param	selectBit; unsigned short:		SPI enable / disable
 * -		SPI_CTL_DISABLE : disable
 * -		SPI_CTL_ENABLE  : enable
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_setComEnable(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= SPI_CH_MAX) ||
		(selectBit > SPI_ENABLE) ) {
			return	SPI_INVALID_PARAMETER;
	}

	// set SPI enable / disable
	RegModify(R16_SPI_CTL(chNo), SPEN, (selectBit << SPEN_SFT));

	return	SPI_SUCCESS;
}

/**
 * @brief	SPI interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	chNo; unsigned char:			channel number
 * -		SPI_CH0 : channel 0
 * @param	intLevel; unsigned char:		interrupt level
 * -		SPI_INT_LEVEL0 : level0
 * -		SPI_INT_LEVEL1 : level1
 * -		SPI_INT_LEVEL2 : level2
 * -		SPI_INT_LEVEL3 : level3
 * -		SPI_INT_LEVEL4 : level4
 * -		SPI_INT_LEVEL5 : level5
 * -		SPI_INT_LEVEL6 : level6
 * -		SPI_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_initInt(unsigned char chNo, unsigned char intLevel)
{
	// parameter check
	if(	(chNo >= SPI_CH_MAX) ||
		(intLevel >= SPI_INT_LEVELMAX) ) {
			return	SPI_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(SPI_INTLVL_REG[chNo], intLevel);

	return	SPI_SUCCESS;
}

/**
 * @brief	SPI interrupt enable / disable.
 * - 1. check parameters.
 * - 2. set an interrupt enable/disable.
 *
 * @param	chNo; unsigned char:			channel number
 * -		SPI_CH0 : channel 0
 * @param	selectBit; unsigned short:		interrupt enable / disable
 * -		SPI_RXINT_DISABLE : receive buffer full interrupt disable
 * -		SPI_RXINT_ENABLE  : receive buffer full interrupt enable
 * -		SPI_TXINT_DISABLE : transmit buffer empty interrupt disable
 * -		SPI_TXINT_ENABLE  : transmit buffer empty interrupt enable
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_controlInt(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= SPI_CH_MAX) ||
		((selectBit & ~SPI_IEN_MASK) != 0x0000) ) {
			return	SPI_INVALID_PARAMETER;
	}

	// set interrupt enable / disable
	RegModify(R16_SPI_CTL(chNo), SPI_IEN_MASK, selectBit);

	return	SPI_SUCCESS;
}

/**
 * @brief	SPI interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 *
 * @param	chNo; unsigned char:			channel number
 * -		SPI_CH0 : channel 0
 * @param	*flag; unsigned short:			check interrupt flag
 * -		SPI_RXFLAG_NOTFULL  : receive data buffer is not full
 * -		SPI_RXFLAG_FULL     : receive data buffer is full
 * -		SPI_TXFLAG_NOTEMPTY : transmit data buffer is not empty
 * -		SPI_TXFLAG_EMPTY    : transmit data buffer is empty
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_checkIntFlag(unsigned char chNo, unsigned short *flag)
{
	// parameter check
	if(chNo >= SPI_CH_MAX) {
		return	SPI_INVALID_PARAMETER;
	}

	// get interrupt flag
	*flag = RegRead(R16_SPI_ST(chNo)) & (SPRBF | SPTBE);

	return	SPI_SUCCESS;
}

/**
 * @brief	cherck a transfer BUSY flag.
 * - 1. check parameters.
 * - 2. read a condition flag and set the condition to "flag".
 *
 * @param	chNo; unsigned char:			channel number
 * -		SPI_CH0 : channel 0
 * @param	*flag; unsigned short:			check flag
 * -		master mode
 * 	-		SPI_BUSYFLAG_IDLE : transfer condition is idle
 * 	-		SPI_BUSYFLAG_BUSY : transfer condition is busy
 * -		slave mode
 * 	-		SPI_SSFLAG_H : slave select signal(xSPISS) high level(not select condition)
 * 	-		SPI_SSFLAG_L : slave select signal(xSPISS) low level(select condition)
 * @return	short:							end status
 * -		SPI_SUCCESS : success
 * -		SPI_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SPI_checkBusyFlag(unsigned char chNo, unsigned short *flag)
{
	// parameter check
	if(chNo >= SPI_CH_MAX) {
		return	SPI_INVALID_PARAMETER;
	}

	// get interrupt flag
	*flag = RegRead(R16_SPI_ST(chNo)) & SPBSY;

	return	SPI_SUCCESS;
}

#endif	// end of PE_SPI
