/**
 * @file	t8osc1.c
 * @brief	8-bit OSC1 timer driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_T8OSC1
#include <t8osc1_api.h>
#include <osc_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short T8O_init(unsigned char mode, unsigned char selectBit, unsigned char data);
void T8O_resetTimer(void);
short T8O_setTimerMode(unsigned char mode);
short T8O_setTimerRun(unsigned char selectBit);
short T8O_getCounterData(unsigned char *data, unsigned char number);
void T8O_setCompareData(unsigned char data);
short T8O_initInt(unsigned char intLevel);
short T8O_controlInt(unsigned char selectBit);
void T8O_resetIntFlag(void);
void T8O_checkIntFlag(unsigned char *flag);
void T8O_setPWMOut(unsigned char data);


/**
 * @brief	8-bit OSC1 timer initialize.
 * - 1. check parameters.
 * - 2. stop a 8-bit OSC1 timer.
 * - 3. set a count mode.
 * - 4. set a T8OSC1 clock division ratio.
 * - 5. set a compare data.
 * - 6. reset a counter.
 *
 * @param	mode; unsigned char: 			select count mode
 * -		T8O_TIMMODE_REPEAT 	: repeat mode
 * -		T8O_TIMMODE_ONESHOT : one shot mode
 * @param	selectBit; unsigned char:		select T8OSC1 clock division ratio
 * -		refer to OSC_setT8OSC1Division
 * @param	data; unsigned char:			compare data
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		T8O_SUCCESS : success
 * -		T8O_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8O_init(unsigned char mode, unsigned char selectBit, unsigned char data)
{
	// parameter check
	if(	(mode > T8O_TIMMODE_ONESHOT) ||
		(selectBit >= CLG_OSC_T8OSC1CLK_MAX) ) {
			return	T8O_INVALID_PARAMETER;
	}

	// 8-bit OSC1 timer stop & mode set
	T8O_setTimerMode(mode);

	// OSC clock division ratio set
	CLG_OSC_setT8OSC1Division(selectBit);

	// compare data set
	T8O_setCompareData(data);

	// 8-bit OSC1 timer reset
	T8O_resetTimer();

	return	T8O_SUCCESS;
}

/**
 * @brief	reset a 8-bit OSC1 timer.
 * - 1. reset a 8-bit OSC1 timer.
 *
 * @return	none
 * @date	2009/12/25
 */
void T8O_resetTimer(void)
{
	// timer reset
	RegModify(R08_T8OSC1_CTL, T8ORST, (T8ORST * T8O_RESET));
}

/**
 * @brief	set a 8-bit OSC1 timer mode.
 * - 1. check parameters.
 * - 2. stop a 8-bit OSC1 timer.
 * - 3. set a 8-bit OSC1 timer mode.
 *
 * @param	mode; unsigned char: 			select count mode
 * -		T8O_TIMMODE_REPEAT 	: repeat mode
 * -		T8O_TIMMODE_ONESHOT : one shot mode
 * @return	short:							end status
 * -		T8O_SUCCESS : success
 * -		T8O_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8O_setTimerMode(unsigned char mode)
{
	// parameter check
	if(mode > T8O_TIMMODE_ONESHOT) {
		return	T8O_INVALID_PARAMETER;
	}

	// clock stop
	T8O_setTimerRun(T8O_TIMER_STOP);

	// set timer mode
	RegModify(R08_T8OSC1_CTL, T8ORMD, (mode << T8ORMD_SFT));

	return	T8O_SUCCESS;
}

/**
 * @brief	set a 8-bit OSC1 timer run / stop.
 * - 1. check parameters.
 * - 2. set a 8-bit OSC1 timer run/stop.
 *
 * @param	selectBit; unsigned char: 		select run / stop
 * -		T8O_TIMER_STOP : 8-bit OSC1 timer stop
 * -		T8O_TIMER_RUN  : 8-bit OSC1 timer run
 * @return	short:							end status
 * -		T8O_SUCCESS : success
 * -		T8O_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8O_setTimerRun(unsigned char selectBit)
{
	// parameter check
	if(selectBit > T8O_TIMER_RUN) {
		return	T8O_INVALID_PARAMETER;
	}

	// timer run / stop
	RegModify(R08_T8OSC1_CTL, T8ORUN, (selectBit << T8ORUN_SFT));

	return	T8O_SUCCESS;
}

/**
 * @brief	get a counter data.
 * - 1. get a counter data from register and set the data to "data".
 * 	-	Note: The correct counter value may not be read out (reading is unstable)
 * 			  if the counter register is read while counting is underway.
 * 	-		  a count value returns success with a continuation match twice
 * 			  and return a failure with a designation number not match.
 *
 * @param	*data; unsigned char:			counter data
 * -		from 0x00 to 0xFF
 * @param	number; unsigned char:			number of a get count value
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		T8O_SUCCESS : success
 * -		T8O_FAILURE : get count value failure
 * @date	2009/12/25
 */
short T8O_getCounterData(unsigned char *data, unsigned char number)
{
	unsigned char	i;														// counter
	unsigned char	cmp_data;												// compare data

	// counter data get
	for(i = 0; i < number; i++) {
		*data    = RegRead(R08_T8OSC1_CNT);									// data read
		cmp_data = RegRead(R08_T8OSC1_CNT);									// compare data read

		if(*data == cmp_data) {
			break;															// data OK
		}
	}

	// data read check
	if(i == number) {
		return	T8O_FAILURE;
	}
	return	T8O_SUCCESS;
}

/**
 * @brief	set a 8-bit OSC1 timer compare data.
 * - 1. set a compare data of 8-bit OSC1 timer.
 *
 * @param	data; unsigned char:			compare data
 * -		from 0x00 to 0xFF
 * @return	none
 * @date	2009/12/25
 */
void T8O_setCompareData(unsigned char data)
{
	// compare data set
	RegWrite(R08_T8OSC1_CMP, data);
}

/**
 * @brief	8-bit OSC1 timer interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		T8O_INT_LEVEL0 : level0
 * -		T8O_INT_LEVEL1 : level1
 * -		T8O_INT_LEVEL2 : level2
 * -		T8O_INT_LEVEL3 : level3
 * -		T8O_INT_LEVEL4 : level4
 * -		T8O_INT_LEVEL5 : level5
 * -		T8O_INT_LEVEL6 : level6
 * -		T8O_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		T8O_SUCCESS : success
 * -		T8O_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8O_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= T8O_INT_LEVELMAX) {
		return	T8O_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(T8O_INTLVL, intLevel);

	return	T8O_SUCCESS;
}

/**
 * @brief	8-bit OSC1 timer interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		T8O_INT_DISABLE : interrupt disable
 * -		T8O_INT_ENABLE  : interrupt enable
 * @return	short:							end status
 * -		T8O_SUCCESS : success
 * -		T8O_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T8O_controlInt(unsigned char selectBit)
{
	// parameter check
	if(selectBit > T8O_INT_ENABLE) {
		return	T8O_INVALID_PARAMETER;
	}

	// reset interrupt flag
	T8O_resetIntFlag();

	// set interrupt enable
	RegWrite(R08_T8OSC1_IMSK, (selectBit << T8OIE_SFT));

	return	T8O_SUCCESS;
}

/**
 * @brief	8-bit OSC1 timer interrupt flag reset.
 * - 1. reset an interrupt status.
 *
 * @return	none
 * @date	2009/12/25
 */
void T8O_resetIntFlag(void)
{
	// reset interrupt flag
	RegWrite(R08_T8OSC1_IFLG, T8OIF);
}

/**
 * @brief	8-bit OSC1 timer interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 *
 * @param	*flag; unsigned char:			check interrupt flag
 * -		T8O_TIMERINT_NOTOCCURRED : interrupt is not occurred
 * -		T8O_TIMERINT_OCCURRED    : interrupt is occurred
 * @return	none
 * @date	2009/12/25
 */
void T8O_checkIntFlag(unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R08_T8OSC1_IFLG);
}

/**
 * @brief	set a PWM output
 * - 1. set a PWM output port.
 * - 2. set a PWM output duty data.
 *
 * @param	data; unsigned char:			PWM output duty data
 * -		from 0x00 to 0xFF
 * @return	none
 * @date	2010/04/15 : Fix MUX_setT8OSC1port.
 * @version
 * -		2009/12/25
 */
void T8O_setPWMOut(unsigned char data)
{
	MUX_USE_T8OSC1PORT use;

	// PWM port set
	use.sw = MUX_ON;
#if	defined(S1C17624) || defined(S1C17622)
	use.useTOUT4 = MUX_TOUT4_P37;
#endif	// end of S1C17624/S1C17622
	MUX_setT8OSC1port(&use);

	// PWM output duty data set
	RegWrite(R08_T8OSC1_DUTY, data);
}

#endif	// end of PE_T8OSC1
